/*******************************************************************
REPLICATION FILE FOR:
Paper: "How Talking Across Ethnic and Class Divides Shapes Support for Ethnic Politics"
Authors: Laura Paler, Leslie Marshall, and Sami Atallah

This do file replicates the randomization inference simulation using the original data (see README).
Note that the simulation takes about 3 hours per variable x 7 variables to complete.
********************************************************************/


/*******************************************************
*WORKING DIRECTORIES
*******************************************************/

*Set your personal working directory here: cd~
cd ~/Dropbox/LEBANON/WRITING/2_DISCUSSION/REPLICATION

	global dos "DOs_and_ADOs"
	global packages "DOs_and_ADOs/PACKAGES"

	global data_orig "DATA/ORIG"
	global data_newimp "DATA/NEW_IMPUTE"
	global working_newimp "DATA/NEW_IMPUTE/WORKING"

	global ri_final "DATA/ORIG/RI"
		
	global results "RESULTS" 
	global results_figs "RESULTS/FIGS"
	global results_tables "RESULTS/TABLES"
	
	set more off

*Call the data actually used in the paper
	use "$data_orig/Leb_3_Clean_FINAL_Orig.dta", clear
	

*Installing version controlled packages
	*The ado packages used and their help files are located in the DOs_and_ADOs/PACKAGES folder. 
	*The following code tells STATA to look for ados in that folder
	*They will then be loaded automatically
	adopath ++ "$packages"
	do "$packages/randomize.ado"
	
*************************************************
*RUN MACROS NEEDED IN ANALYSIS
*RI done for 'big' strata so based on wgt6
*************************************************
	
	global demo_minus_gen pre_q2_age scr_q3_marital_REC2 scr_q4_educ_REC2 
	global sect_minus_sun pre_q3_christian pre_q3_shia 
	global econ_id pre_q14_d_class_REC1b 
	global other_id pre_q14_leb_REC1b	
	global implement mod1 n_gid post_q7_knowppl_REC1 
	
	global econ_wgt6 index_econ_all_wgt6 scr_q6_work_REC1  
	global prej_wgt6 index_prej_wgt6 
	global act_wgt6 index_act_wgt6
	global networks_wgt6 index_soc_hom_wgt6 pre_q13_disagree_REC1 
	global sect_id_wgt6 index_sect_id_wgt6
	global connect_wgt6 index_connect_wgt6
	
	global controls_all_wgt6 $demo_minus_gen $sect_minus_sun $econ_wgt6 $prej_wgt6 $act_wgt6 $networks_wgt6 $sect_id_wgt6 $connect_wgt6 $econ_id $other_id $implement 
		
*************************************************
*PREP
*************************************************
			
	keep pet_allsign me_beirut_REC1 me_poor index_y_learn_shared_wgt4 index_y_press_wgt4 index_y_agree_wgt4 q8_id_diff_sect_diff_class $controls_all_wgt6 group2 group3 group4 strata_agg_FINAL strata_agg_FINAL2 pid gid m_impute wgt6 

	keep if m_impute==1

	g one = 1

	bysort strata_agg_FINAL: egen n_in_block = count(one)
		
*************************************************
*PROGRAM FOR SIMULATION
*************************************************

capture prog drop lebsim

program define lebsim, rclass
    version 11
   		syntax varlist [if]
        marksample touse
        gettoken y : varlist
        set more off
        preserve
        keep if  `touse'


*Obtain real results
	qui: areg `y' group2 group3 group4 [pweight=wgt6], absorb(strata_agg_FINAL) vce(cluster gid)
     return scalar b_y_1r=_b[group2]
     return scalar b_y_2r=_b[group3]
     return scalar b_y_3r=_b[group4]

	
*Random reassignment	
	randomize , groups(4) block(strata_agg_FINAL) replace generate(ri_assign)

	bysort strata_agg_FINAL ri_assign: egen n_in_cell = count(one)
	
	g ri_wgt_pop = 			 9/n_in_cell  if ri_assign==1
		replace ri_wgt_pop = 9/n_in_cell  if ri_assign==2
		replace ri_wgt_pop = 10/n_in_cell if ri_assign==3
		replace ri_wgt_pop = 12/n_in_cell if ri_assign==4
	
*Assignment variables	
	g ri_group2 = ri_assign==2
	g ri_group3 = ri_assign==3
	g ri_group4 = ri_assign==4
		
*Regression
	qui: areg `y' ri_group2 ri_group3 ri_group4 [pweight=ri_wgt_pop], absorb(strata_agg_FINAL) vce(cluster gid)
	 return scalar b_y_1s=_b[ri_group2]
     return scalar b_y_2s=_b[ri_group3]
     return scalar b_y_3s=_b[ri_group4]

	
*Prep for next round	
	drop n_in_cell ri_*

	
end
	
	
*************************************************
*RUN SIMULATION
*Run all code below in the loops
*Inputs list of outcome variables
*Runs simulation
*Calculates p-values from two-tailed test
*Saves dataset with 10,000 sims and p-values
*************************************************

version 15.0
sort m_impute pid
	

		foreach var of varlist pet_allsign me_beirut_REC1 me_poor index_y_learn_shared_wgt4 index_y_agree_wgt4 q8_id_diff_sect_diff_class index_y_press_wgt4 { 
				preserve
				simulate b_y_1r=r(b_y_1r) b_y_1s=r(b_y_1s) b_y_2r=r(b_y_2r) b_y_2s=r(b_y_2s) b_y_3r=r(b_y_3r) b_y_3s=r(b_y_3s), reps(10000) saving("$ri_final/lebsim_`var'.dta", replace) dots(100) seed(20190618): lebsim `var'
		
		*Calculate two-tailed tests
			foreach num of numlist 1 2 3 {
					g pvi_`num'_tt = abs(b_y_`num's) > abs(b_y_`num'r)
					egen pv_`num'_tt = mean(pvi_`num'_tt)
				}		
	
				save "$ri_final/lebsim_`var'.dta", replace
				restore
		}	
		
/*************************************************
*CALCULATE ONE-TAILED TESTS
If b_y_1r is positive, want to know probability that b_1_ys > b_1_yr (same for b_y_2r and b_y_3r)
If b_y_1r is negative, want to know probability that b_1_ys < b_1_yr (same for b_y_2r and b_y_3r)
*************************************************/
	
*PETITION (b_y_1r, 2r, 3r all positive)	
	use "$ri_final/lebsim_pet_allsign.dta", clear
		foreach num of numlist 1 2 3 {
					g pvi_`num'_ot = b_y_`num's > b_y_`num'r
					egen pv_`num'_ot = mean(pvi_`num'_ot)
				}				
	
	drop pvi*
	save "$ri_final/lebsim_pet_allsign.dta", replace
	
	
*MAP ALLOC TO BEIRUT (b_y_1r, 2r, 3r all negative)	
	use "$ri_final/lebsim_me_beirut_REC1.dta", clear
		foreach num of numlist 1 2 3 {
			g pvi_`num'_ot = b_y_`num's < b_y_`num'r
			egen pv_`num'_ot = mean(pvi_`num'_ot)
	}
	
	drop pvi*
	save "$ri_final/lebsim_me_beirut_REC1.dta", replace	
		
		
*MAP ALLOC TO POOR DISTRICTS (b_y_1r is positive, _2r and _3r are neg)		
	use "$ri_final/lebsim_me_poor.dta", clear
		
	g pvi_1_ot = b_y_1s > b_y_1r
	egen pv_1_ot = mean(pvi_1_ot)

	foreach num of numlist 2 3 {
		g pvi_`num'_ot = b_y_`num's < b_y_`num'r
		egen pv_`num'_ot = mean(pvi_`num'_ot)
	}
		
	drop pvi*
	save "$ri_final/lebsim_me_poor.dta", replace
	

*LEARNING INDEX (1r and 3r are positive, 2r is negative)	
	use "$ri_final/lebsim_index_y_learn_shared_wgt4.dta", clear
	
	foreach num of numlist 1 3 {
		g pvi_`num'_ot = b_y_`num's > b_y_`num'r
		egen pv_`num'_ot = mean(pvi_`num'_ot)
	}
	
	g pvi_2_ot = b_y_2s < b_y_2r
	egen pv_2_ot = mean(pvi_2_ot)
	
	drop pvi*
	save "$ri_final/lebsim_index_y_learn_shared_wgt4.dta", replace
	
	
*AGREEMENT INDEX (1r is positive, 2r and 3r negative)	
	use "$ri_final/lebsim_index_y_agree_wgt4.dta", clear

	g pvi_1_ot = b_y_1s > b_y_1r
	egen pv_1_ot = mean(pvi_1_ot)

	foreach num of numlist 2 3 {
		g pvi_`num'_ot = b_y_`num's < b_y_`num'r
		egen pv_`num'_ot = mean(pvi_`num'_ot)
	}
		
	drop pvi*
	
	save "$ri_final/lebsim_index_y_agree_wgt4.dta", replace
	
*SOCIAL IDENTITY (1r and 3r are positive, 2r is negative)
	use "$ri_final/lebsim_q8_id_diff_sect_diff_class.dta", clear
	
	foreach num of numlist 1 3 {
		g pvi_`num'_ot = b_y_`num's > b_y_`num'r
		egen pv_`num'_ot = mean(pvi_`num'_ot)
	}
	
	g pvi_2_ot = b_y_2s < b_y_2r
	egen pv_2_ot = mean(pvi_2_ot)
	
	drop pvi*
	save "$ri_final/lebsim_q8_id_diff_sect_diff_class.dta", replace	
	
	
*PRESSURE (all are negative)	
	use "$ri_final/lebsim_index_y_press_wgt4.dta", clear
	
	foreach num of numlist 1 2 3 {
		g pvi_`num'_ot = b_y_`num's < b_y_`num'r
		egen pv_`num'_ot = mean(pvi_`num'_ot)
	}
	
	drop pvi*
	save "$ri_final/lebsim_index_y_press_wgt4.dta", replace
	
		
*************************************************
*GRAPHING RESULTS
*************************************************
		
*PETITION RESULTS		
use "$ri_final/lebsim_pet_allsign.dta", clear
	hist b_y_1s, scheme(lean1) xline(.0819048, lcolor(red)) bin(100) xtitle ("p=.049 (one-tailed)") xlab(.08) title("Petition (Group 2 vs. 1)") saving("$results_figs/leb_ri_pet_group2", replace)
	hist b_y_2s, scheme(lean1) xline(.0466667, lcolor(red)) bin(100) xtitle ("p=.176 (one-tailed)") xlab(.05) title("Petition (Group 3 vs. 1)") saving("$results_figs/leb_ri_pet_group3", replace)
	hist b_y_3s, scheme(lean1) xline(.0516667, lcolor(red)) bin(100) xtitle ("p=.151 (one-tailed)") xlab(.05) title("Petition (Group 4 vs. 1)") saving("$results_figs/leb_ri_pet_group4", replace)
		
graph combine "$results_figs/leb_ri_pet_group2" "$results_figs/leb_ri_pet_group3" "$results_figs/leb_ri_pet_group4", ///
	col(3) row(1) scheme(lean1) graphregion(fcolor(white)) saving("$results_figs/leb_ri_petition.gph", replace) iscale(1) ///
	xcommon ycommon altshrink

	
*MAP RESULTS		
use "$ri_final/lebsim_me_beirut_REC1.dta", clear
	hist b_y_1s, scheme(lean1) xline(-.0472398, lcolor(red)) bin(100) xtitle ("p=.000 (one-tailed)") xlab(-.05) subtitle("Beirut (Group 2 vs. 1)") saving("$results_figs/leb_ri_beirut_group2", replace)
	hist b_y_2s, scheme(lean1) xline(-.0115934, lcolor(red)) bin(100) xtitle ("p=.209 (one-tailed)") xlab(-.01) subtitle("Beirut (Group 3 vs. 1)") saving("$results_figs/leb_ri_beirut_group3", replace)
	hist b_y_3s, scheme(lean1) xline(-.0169738, lcolor(red)) bin(100) xtitle ("p=.114 (one-tailed)") xlab(-.02) subtitle("Beirut (Group 4 vs. 1)") saving("$results_figs/leb_ri_beirut_group4", replace)
		
use "$ri_final/lebsim_me_poor.dta", clear
	hist b_y_1s, scheme(lean1) xline(.0479105, lcolor(red)) bin(100) xtitle ("p=.025 (one-tailed)") xlab(.05) subtitle("Poor Districts (Group 2 vs. 1)") saving("$results_figs/leb_ri_poor_group2", replace)
	hist b_y_2s, scheme(lean1) xline(-.011528, lcolor(red)) bin(100) xtitle ("p=.307 (one-tailed)") xlab(-.01) subtitle("Poor Districts (Group 3 vs. 1)") saving("$results_figs/leb_ri_poor_group3", replace)
	hist b_y_3s, scheme(lean1) xline(-.0011908, lcolor(red)) bin(100) xtitle ("p=.476 (one-tailed)") xlab(-.00) subtitle("Poor Districts (Group 4 vs. 1)") saving("$results_figs/leb_ri_poor_group4", replace)
		
graph combine "$results_figs/leb_ri_beirut_group2" "$results_figs/leb_ri_beirut_group3" "$results_figs/leb_ri_beirut_group4" ///
	"$results_figs/leb_ri_poor_group2" "$results_figs/leb_ri_poor_group3" "$results_figs/leb_ri_poor_group4", ///
	col(3) row(2) scheme(lean1) graphregion(fcolor(white)) saving("$results_figs/leb_ri_map.gph", replace)
	
	
*LEARNING RESULTS	
use "$ri_final/lebsim_index_y_learn_shared_wgt4.dta", clear
	hist b_y_1s, scheme(lean1) xline(.261867, lcolor(red)) bin(100) xtitle ("p=.005 (one-tailed)") xlab(.26) subtitle("Learn Shared Prefs (Group 2 vs. 1)") saving("$results_figs/leb_ri_shared_group2", replace)
	hist b_y_2s, scheme(lean1) xline(-.0321107, lcolor(red)) bin(100) xtitle ("p=.307 (one-tailed)") xlab(-.03) subtitle("Learn Shared Prefs (Group 3 vs. 1)") saving("$results_figs/leb_ri_shared_group3", replace)
	hist b_y_3s, scheme(lean1) xline(.0530635, lcolor(red)) bin(100) xtitle ("p=.365 (one-tailed)") xlab(.05) subtitle("Learn Shared Prefs (Group 4 vs. 1)") saving("$results_figs/leb_ri_shared_group4", replace)
			
use "$ri_final/lebsim_index_y_agree_wgt4.dta",  clear
	hist b_y_1s, scheme(lean1) xline(.1988434, lcolor(red)) bin(100) xtitle ("p=.027 (one-tailed)") xlab(.20) subtitle("Agreement Index (Group 2 vs. 1)") saving("$results_figs/leb_ri_agree_group2", replace)
	hist b_y_2s, scheme(lean1) xline(-.16395, lcolor(red)) bin(100) xtitle ("p=.055 (one-tailed)") xlab(-.16) subtitle("Agreement Index (Group 3 vs. 1)") saving("$results_figs/leb_ri_agree_group3", replace)
	hist b_y_3s, scheme(lean1) xline(-.0901422, lcolor(red)) bin(100) xtitle ("p=.194 (one-tailed)") xlab(-.09) subtitle("Agreement Index (Group 4 vs. 1)") saving("$results_figs/leb_ri_agree_group4", replace)
	
graph combine "$results_figs/leb_ri_shared_group2" "$results_figs/leb_ri_shared_group3" "$results_figs/leb_ri_shared_group4" ///
	"$results_figs/leb_ri_agree_group2" "$results_figs/leb_ri_agree_group3" "$results_figs/leb_ri_agree_group4", ///
	col(3) row(2) scheme(lean1) xcommon ycommon graphregion(fcolor(white)) saving("$results_figs/leb_ri_learn_shared.gph", replace)
	
	
*IDENTITY AND SOCIAL PRESSURE RESULTS	
use "$ri_final/lebsim_q8_id_diff_sect_diff_class.dta", clear
	hist b_y_1s, scheme(lean1) xline(.1770635, lcolor(red)) bin(100) xtitle ("p=.194 (one-tailed)") xlab(.18) subtitle("Double Out-Group ID (Group 2 vs. 1)") saving("$results_figs/leb_ri_id_group2", replace)
	hist b_y_2s, scheme(lean1) xline(-.0588889, lcolor(red)) bin(100) xtitle ("p=.382 (one-tailed)") xlab(-.06) subtitle("Double Out-Group ID (Group 3 vs. 1)") saving("$results_figs/leb_ri_id_group3", replace)
	hist b_y_3s, scheme(lean1) xline(.6543056, lcolor(red)) bin(100) xtitle ("p=.001 (one-tailed)") xlab(.65) subtitle("Double Out-Group ID (Group 4 vs. 1)") saving("$results_figs/leb_ri_id_group4", replace)
		
use "$ri_final/lebsim_index_y_press_wgt4.dta", clear
	hist b_y_1s, scheme(lean1) xline(-.1966916, lcolor(red)) bin(100) xtitle ("p=.029 (one-tailed)") xlab(-.20) subtitle("Pressure Index (Group 2 vs. 1)") saving("$results_figs/leb_ri_press_group2", replace)
	hist b_y_2s, scheme(lean1) xline(.0374065, lcolor(red)) bin(100) xtitle ("p=.635 (one-tailed)") xlab(.04) subtitle("Pressure Index (Group 3 vs. 1)") saving("$results_figs/leb_ri_press_group3", replace)
	hist b_y_3s, scheme(lean1) xline(-.257206, lcolor(red)) bin(100) xtitle ("p=.006 (one-tailed)") xlab(-.26) subtitle("Pressure Index (Group 4 vs. 1)") saving("$results_figs/leb_ri_press_group4", replace)
		
			
graph combine "$results_figs/leb_ri_id_group2" "$results_figs/leb_ri_id_group3" "$results_figs/leb_ri_id_group4" ///
	"$results_figs/leb_ri_press_group2" "$results_figs/leb_ri_press_group3" "$results_figs/leb_ri_press_group4", ///
	col(3) row(2) scheme(lean1) graphregion(fcolor(white)) saving("$results_figs/leb_ri_id_press.gph", replace)
		
		
		
		
		
		
		
		
		
